@@ -118,10 +118,12 @@ static Display *X11display = 0;
 static int X11_startup_notify_fd = -1;
 static Display *X11_startup_notify_display = 0;
 #endif
-#define MAX_SOCK_FILE 255
+// Finds size of sun_path without allocating a sockaddr_un to do it.
+// Assumes sun_path is at end of sockaddr_un though
+#define MAX_SOCK_FILE (sizeof(struct sockaddr_un) - offsetof(struct sockaddr_un,sun_path))
 static char sock_file[MAX_SOCK_FILE];
 
-static QByteArray displayEnvVarName()
+static const char* displayEnvVarName_c()
 {
     // Can't use QGuiApplication::platformName() here, there is no app instance.
 #if HAVE_X11
@@ -132,6 +134,10 @@ static QByteArray displayEnvVarName()
     return "WIN_DISPLAY";
 #endif
 }
+static inline QByteArray displayEnvVarName()
+{
+    return QByteArray::fromRawData(displayEnvVarName_c(), strlen(displayEnvVarName_c()));
+}
 
 /* Group data */
 static struct {
@@ -406,7 +412,7 @@ static void complete_startup_info(KStartupInfoId &id, pid_t pid)
 QByteArray execpath_avoid_loops(const QByteArray &exec, int envc, const char *envs, bool avoid_loops)
 {
     QStringList paths;
-    const QRegExp pathSepRegExp(QString::fromLatin1("[:\b]"));
+    const QRegExp pathSepRegExp(QStringLiteral("[:\b]"));
     if (envc > 0) { /* use the passed environment */
         const char *path = get_env_var("PATH=", envc, envs);
         if (path != NULL) {
@@ -651,7 +657,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
             exitWithErrorMsg(errorMsg);
         }
 
-        if (!qgetenv("KDE_IS_PRELINKED").isEmpty() && !execpath.isEmpty()) {
+        if (!qEnvironmentVariableIsEmpty("KDE_IS_PRELINKED") && !execpath.isEmpty()) {
             libpath.truncate(0);
         }
 
@@ -922,7 +928,7 @@ static void init_kdeinit_socket()
             exit(255);
         }
         server.sun_family = AF_UNIX;
-        strcpy(server.sun_path, sock_file);
+        qstrncpy(server.sun_path, sock_file, sizeof(server.sun_path));
         socklen = sizeof(server);
 
         if (connect(s, (struct sockaddr *)&server, socklen) == 0) {
@@ -964,7 +970,7 @@ static void init_kdeinit_socket()
         socklen = sizeof(sa);
         memset(&sa, 0, socklen);
         sa.sun_family = AF_UNIX;
-        strcpy(sa.sun_path, sock_file);
+        qstrncpy(sa.sun_path, sock_file, sizeof(sa.sun_path));
         if (bind(d.wrapper, (struct sockaddr *)&sa, socklen) != 0) {
             if (max_tries == 0) {
                 perror("kdeinit5: Aborting. bind() failed");
@@ -1073,7 +1079,8 @@ static bool handle_launcher_request(int sock, const char *who)
     }
 
     if (request_header.arg_length != 0) {
-        request_data = (char *) malloc(request_header.arg_length);
+        request_data = (char *) malloc(request_header.arg_length + 1);
+        request_data[request_header.arg_length] = '\0';
 
         result = read_socket(sock, request_data, request_header.arg_length);
         if (result != 0) {
@@ -1165,14 +1172,14 @@ static bool handle_launcher_request(int sock, const char *who)
         }
 
         // support for the old a bit broken way of setting DISPLAY for multihead
-        QByteArray olddisplay = qgetenv(displayEnvVarName().constData());
+        QByteArray olddisplay = qgetenv(displayEnvVarName_c());
         QByteArray kdedisplay = qgetenv("KDE_DISPLAY");
         bool reset_display = (! olddisplay.isEmpty() &&
                               ! kdedisplay.isEmpty() &&
                               olddisplay != kdedisplay);
 
         if (reset_display) {
-            qputenv(displayEnvVarName().constData(), kdedisplay);
+            qputenv(displayEnvVarName_c(), kdedisplay);
         }
 
         pid = launch(argc, name, args, cwd, envc, envs,
@@ -1181,7 +1188,7 @@ static bool handle_launcher_request(int sock, const char *who)
 
         if (reset_display) {
             unsetenv("KDE_DISPLAY");
-            qputenv(displayEnvVarName().constData(), olddisplay);
+            qputenv(displayEnvVarName_c(), olddisplay);
         }
 
         if (pid && (d.result == 0)) {
@@ -1391,10 +1398,10 @@ static void handle_requests(pid_t waitForPid)
 static void generate_socket_name()
 {
 
-    QByteArray display = qgetenv(displayEnvVarName().constData());
+    QByteArray display = qgetenv(displayEnvVarName_c());
     if (display.isEmpty()) {
-#if HAVE_X11 // qt5: see displayEnvVarName()
-        fprintf(stderr, "kdeinit5: Aborting. $%s is not set. \n", displayEnvVarName().constData());
+#if HAVE_X11 // qt5: see displayEnvVarName_c()
+        fprintf(stderr, "kdeinit5: Aborting. $%s is not set. \n", displayEnvVarName_c());
         exit(255);
 #endif
     }
@@ -1408,9 +1415,9 @@ static void generate_socket_name()
     display.replace('/', '_');
 #endif
     // WARNING, if you change the socket name, adjust kwrapper too
-    const QString socketFileName = QString::fromLatin1("kdeinit5_%1").arg(QLatin1String(display));
-    QByteArray socketName = QFile::encodeName(QStandardPaths::writableLocation(QStandardPaths::RuntimeLocation) + QLatin1Char('/') + socketFileName);
-    if (socketName.length() >= MAX_SOCK_FILE) {
+    const QString socketFileName = QStringLiteral("kdeinit5_%1").arg(QLatin1String(display));
+    const QByteArray socketName = QFile::encodeName(QStandardPaths::writableLocation(QStandardPaths::RuntimeLocation) + QLatin1Char('/') + socketFileName);
+    if (static_cast<unsigned>(socketName.length()) >= MAX_SOCK_FILE) {
         fprintf(stderr, "kdeinit5: Aborting. Socket name will be too long:\n");
         fprintf(stderr, "         '%s'\n", socketName.data());
         exit(255);
@@ -1512,8 +1519,8 @@ static void setupX()
         as well, point XAUTHORITY there and never remove the file (except for possible
         tmp cleanup).
     */
-    if (!qgetenv("XAUTHORITY").isEmpty()) {
-        QByteArray display = qgetenv(displayEnvVarName().constData());
+    if (!qEnvironmentVariableIsEmpty("XAUTHORITY")) {
+        QByteArray display = qgetenv(displayEnvVarName_c());
         int i;
         if ((i = display.lastIndexOf('.')) > display.lastIndexOf(':') && i >= 0) {
             display.truncate(i);
@@ -1523,7 +1530,7 @@ static void setupX()
         display.replace('/', '_');
 #endif
         QString xauth = QDir::tempPath() + QLatin1String("/xauth-")
-                        + QString::number(getuid()) + QLatin1String("-") + QString::fromLocal8Bit(display);
+                        + QString::number(getuid()) + QLatin1Char('-') + QString::fromLocal8Bit(display);
         QSaveFile xauthfile(xauth);
         QFile xauthfrom(QFile::decodeName(qgetenv("XAUTHORITY")));
         if (!xauthfrom.open(QFile::ReadOnly) || !xauthfile.open(QFile::WriteOnly)
@@ -1765,7 +1772,7 @@ int main(int argc, char **argv)
         init_kdeinit_socket();
     }
 #ifdef Q_OS_UNIX
-    if (!d.suicide && qgetenv("KDE_IS_PRELINKED").isEmpty()) {
+    if (!d.suicide && qEnvironmentVariableIsEmpty("KDE_IS_PRELINKED")) {
         const int extrasCount = sizeof(extra_libs) / sizeof(extra_libs[0]);
         for (int i = 0; i < extrasCount; i++) {
             const QString extra = findSharedLib(QString::fromLatin1(extra_libs[i]));
