@@ -67,6 +67,16 @@
 
 // #define KLAUNCHER_VERBOSE_OUTPUT
 
+#include <QLoggingCategory>
+Q_DECLARE_LOGGING_CATEGORY(KLAUNCHER)
+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
+// logging category for this framework, default: log stuff >= warning
+Q_LOGGING_CATEGORY(KLAUNCHER, "kf5.kinit.klauncher", QtWarningMsg)
+#else
+Q_LOGGING_CATEGORY(KLAUNCHER, "kf5.kinit.klauncher")
+#endif
+
 static const char *const s_DBusStartupTypeToString[] =
 { "DBusNone", "DBusUnique", "DBusMulti", "ERROR" };
 
@@ -109,7 +119,7 @@ KLauncher::KLauncher()
     mAutoTimer.setSingleShot(true);
     new KLauncherAdaptor(this);
     mSlaveLauncherAdaptor = new KSlaveLauncherAdaptor(this);
-    QDBusConnection::sessionBus().registerObject(QLatin1String("/KLauncher"), this); // same as ktoolinvocation.cpp
+    QDBusConnection::sessionBus().registerObject(QStringLiteral("/KLauncher"), this); // same as ktoolinvocation.cpp
 
     connect(&mAutoTimer, SIGNAL(timeout()), this, SLOT(slotAutoStart()));
     connect(QDBusConnection::sessionBus().interface(),
@@ -120,7 +130,7 @@ KLauncher::KLauncher()
     connect(&mConnectionServer, SIGNAL(newConnection()), SLOT(acceptSlave()));
     if (!mConnectionServer.isListening()) {
         // Severe error!
-        qDebug("KLauncher: Fatal error, can't create tempfile!");
+        qWarning("KLauncher: Fatal error, can't create tempfile!");
         ::_exit(1);
     }
 
@@ -153,7 +163,7 @@ KLauncher::KLauncher()
         qWarning("Klauncher running slaves through valgrind for slaves of protocol '%s'", qPrintable(mSlaveValgrind));
     }
 #ifdef USE_KPROCESS_FOR_KIOSLAVES
-    // qDebug() << "LAUNCHER_OK";
+    qCDebug(KLAUNCHER) << "LAUNCHER_OK";
 #else
     klauncher_header request_header;
     request_header.cmd = LAUNCHER_OK;
@@ -228,7 +238,7 @@ read_socket(int sock, char *buffer, int len)
         FD_SET(sock, &in);
         select(sock + 1, &in, 0, 0, &tm);
         if (!FD_ISSET(sock, &in)) {
-            // qDebug() << "read_socket" << sock << "nothing to read, kdeinit5 must be dead";
+            qCDebug(KLAUNCHER) << "read_socket" << sock << "nothing to read, kdeinit5 must be dead";
             return -1;
         }
 
@@ -256,7 +266,7 @@ KLauncher::slotKDEInitData(int)
     int result = read_socket(kdeinitSocket, (char *) &request_header,
                              sizeof(request_header));
     if (result == -1) {
-        // qDebug() << "Exiting on read_socket errno:" << errno;
+        qCDebug(KLAUNCHER) << "Exiting on read_socket errno:" << errno;
         signal(SIGHUP, SIG_IGN);
         signal(SIGTERM, SIG_IGN);
         destruct(); // Exit!
@@ -281,7 +291,7 @@ void KLauncher::processRequestReturn(int status, const QByteArray &requestData)
         long *request_data;
         request_data = (long *) requestData.data();
         lastRequest->pid = (pid_t)(*request_data);
-        // qDebug().nospace() << lastRequest->name << " (pid " << lastRequest->pid << ") up and running.";
+        qCDebug(KLAUNCHER).nospace() << lastRequest->name << " (pid " << lastRequest->pid << ") up and running.";
         switch (lastRequest->dbus_startup_type) {
         case KService::DBusNone:
             if (lastRequest->wait) {
@@ -300,7 +310,7 @@ void KLauncher::processRequestReturn(int status, const QByteArray &requestData)
     }
     if (lastRequest && (status == LAUNCHER_ERROR)) {
         lastRequest->status = KLaunchRequest::Error;
-        // qDebug() << lastRequest->name << " failed." << endl;
+        qCDebug(KLAUNCHER) << lastRequest->name << " failed.";
         if (!requestData.isEmpty()) {
             lastRequest->errorMsg = QString::fromUtf8((char *) requestData.data());
         }
@@ -315,31 +325,31 @@ void
 KLauncher::processDied(pid_t pid, long exitStatus)
 {
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-    // qDebug() << pid << "exitStatus=" << exitStatus;
+    qCDebug(KLAUNCHER) << pid << "exitStatus=" << exitStatus;
 #else
     Q_UNUSED(exitStatus);
     // We should probably check the exitStatus for the uniqueapp case?
 #endif
     foreach (KLaunchRequest *request, requestList) {
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-        // qDebug() << "  had pending request" << request->pid;
+        qCDebug(KLAUNCHER) << "  had pending request" << request->pid;
 #endif
         if (request->pid == pid) {
             if ((request->dbus_startup_type == KService::DBusUnique)
                     && QDBusConnection::sessionBus().interface()->isServiceRegistered(request->dbus_name)) {
                 request->status = KLaunchRequest::Running;
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-                // qDebug() << pid << "running as a unique app";
+                qCDebug(KLAUNCHER) << pid << "running as a unique app";
 #endif
             } else if(request->dbus_startup_type == KService::DBusNone && request->wait) {
                 request->status = KLaunchRequest::Running;
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-                // qDebug() << pid << "running as DBusNone with wait to true";
+                qCDebug(KLAUNCHER) << pid << "running as DBusNone with wait to true";
 #endif
             } else {
                 request->status = KLaunchRequest::Error;
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-                // qDebug() << pid << "died, requestDone. status=" << request->status;
+                qCDebug(KLAUNCHER) << pid << "died, requestDone. status=" << request->status;
 #endif
             }
             requestDone(request);
@@ -347,7 +357,7 @@ KLauncher::processDied(pid_t pid, long exitStatus)
         }
     }
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-    // qDebug() << "found no pending requests for PID" << pid;
+    qCDebug(KLAUNCHER) << "found no pending requests for PID" << pid;
 #endif
 }
 
@@ -358,12 +368,12 @@ static bool matchesPendingRequest(const QString &appId, const QString &pendingAp
 
     const QString newAppId = appId.left(appId.lastIndexOf(QLatin1Char('-'))); // strip out the -12345 if present.
 
-    //qDebug() << "appId=" << appId << "newAppId=" << newAppId << "pendingAppId=" << pendingAppId;
+    qCDebug(KLAUNCHER) << "appId=" << appId << "newAppId=" << newAppId << "pendingAppId=" << pendingAppId;
 
     if (pendingAppId.startsWith(QLatin1String("*."))) {
         const QString pendingName = pendingAppId.mid(2);
         const QString appName = newAppId.mid(newAppId.lastIndexOf(QLatin1Char('.')) + 1);
-        //qDebug() << "appName=" << appName;
+        qCDebug(KLAUNCHER) << "appName=" << appName;
         return appName == pendingName;
     }
 
@@ -380,7 +390,7 @@ KLauncher::slotNameOwnerChanged(const QString &appId, const QString &oldOwner,
     }
 
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-    // qDebug() << "new app" << appId;
+    qCDebug(KLAUNCHER) << "new app" << appId;
 #endif
     foreach (KLaunchRequest *request, requestList) {
         if (request->status != KLaunchRequest::Launching) {
@@ -388,7 +398,7 @@ KLauncher::slotNameOwnerChanged(const QString &appId, const QString &oldOwner,
         }
 
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-        // qDebug() << "had pending request" << request->name << s_DBusStartupTypeToString[request->dbus_startup_type] << "dbus_name" << request->dbus_name << request->tolerant_dbus_name;
+        qCDebug(KLAUNCHER) << "had pending request" << request->name << s_DBusStartupTypeToString[request->dbus_startup_type] << "dbus_name" << request->dbus_name << request->tolerant_dbus_name;
 #endif
         // For unique services check the requested service name first
         if (request->dbus_startup_type == KService::DBusUnique) {
@@ -396,20 +406,20 @@ KLauncher::slotNameOwnerChanged(const QString &appId, const QString &oldOwner,
                     QDBusConnection::sessionBus().interface()->isServiceRegistered(request->dbus_name)) { // was already running
                 request->status = KLaunchRequest::Running;
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-                // qDebug() << "OK, unique app" << request->dbus_name << "is running";
+                qCDebug(KLAUNCHER) << "OK, unique app" << request->dbus_name << "is running";
 #endif
                 requestDone(request);
                 continue;
             } else {
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-                // qDebug() << "unique app" << request->dbus_name << "not running yet";
+                qCDebug(KLAUNCHER) << "unique app" << request->dbus_name << "not running yet";
 #endif
             }
         }
 
         const QString rAppId = !request->tolerant_dbus_name.isEmpty() ? request->tolerant_dbus_name : request->dbus_name;
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-        //qDebug() << "using" << rAppId << "for matching";
+        qCDebug(KLAUNCHER) << "using" << rAppId << "for matching";
 #endif
         if (rAppId.isEmpty()) {
             continue;
@@ -417,7 +427,7 @@ KLauncher::slotNameOwnerChanged(const QString &appId, const QString &oldOwner,
 
         if (matchesPendingRequest(appId, rAppId)) {
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-            // qDebug() << "ok, request done";
+            qCDebug(KLAUNCHER) << "ok, request done";
 #endif
             request->dbus_name = appId;
             request->status = KLaunchRequest::Running;
@@ -446,6 +456,7 @@ KLauncher::slotAutoStart()
     KService::Ptr s;
     do {
         QString service = mAutoStart.startService();
+        qCDebug(KLAUNCHER) << "Service: " << mAutoStart.phase() << service;
         if (service.isEmpty()) {
             // Done
             if (!mAutoStart.phaseDone()) {
@@ -476,14 +487,14 @@ KLauncher::requestDone(KLaunchRequest *request)
             (request->status == KLaunchRequest::Done)) {
         requestResult.result = 0;
         requestResult.dbusName = request->dbus_name;
-        requestResult.error = QString::fromLatin1(""); // not null, cf assert further down
+        requestResult.error = QStringLiteral(""); // not null, cf assert further down
         requestResult.pid = request->pid;
     } else {
         requestResult.result = 1;
         requestResult.dbusName.clear();
         requestResult.error = i18n("KDEInit could not launch '%1'", request->name);
         if (!request->errorMsg.isEmpty()) {
-            requestResult.error += QString::fromLatin1(":\n") + request->errorMsg;
+            requestResult.error += QStringLiteral(":\n") + request->errorMsg;
         }
         requestResult.pid = 0;
 
@@ -526,7 +537,7 @@ KLauncher::requestDone(KLaunchRequest *request)
                                            << stream_pid));
     }
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-    // qDebug() << "removing done request" << request->name << "PID" << request->pid;
+    qCDebug(KLAUNCHER) << "removing done request" << request->name << "PID" << request->pid;
 #endif
 
     requestList.removeAll(request);
@@ -617,7 +628,7 @@ KLauncher::requestStart(KLaunchRequest *request)
     request_header.arg_length = requestData.length();
 
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-    // qDebug() << "Asking kdeinit to start" << request->name << request->arg_list
+    qCDebug(KLAUNCHER) << "Asking kdeinit to start" << request->name << request->arg_list
             << "cmd=" << commandToString(request_header.cmd);
 #endif
 
@@ -747,7 +758,7 @@ KLauncher::start_service(KService::Ptr service, const QStringList &_urls,
     createArgs(request, service, qurls);
 
     // We must have one argument at least!
-    if (!request->arg_list.count()) {
+    if (request->arg_list.isEmpty()) {
         requestResult.result = ENOEXEC;
         requestResult.error = i18n("Service '%1' is malformatted.", service->entryPath());
         delete request;
@@ -763,26 +774,26 @@ KLauncher::start_service(KService::Ptr service, const QStringList &_urls,
         // Testcase: konqueror www.kde.org, RMB on link, open with, kruler.
 
         request->dbus_startup_type = KService::DBusMulti;
-        request->dbus_name = QString::fromLatin1("org.kde.kioexec");
+        request->dbus_name = QStringLiteral("org.kde.kioexec");
     } else {
         request->dbus_startup_type = service->dbusStartupType();
 
         if ((request->dbus_startup_type == KService::DBusUnique) ||
                 (request->dbus_startup_type == KService::DBusMulti)) {
-            const QVariant v = service->property(QLatin1String("X-DBUS-ServiceName"));
+            const QVariant v = service->property(QStringLiteral("X-DBUS-ServiceName"));
             if (v.isValid()) {
                 request->dbus_name = v.toString();
             }
             if (request->dbus_name.isEmpty()) {
                 const QString binName = KIO::DesktopExecParser::executableName(service->exec());
-                request->dbus_name = QString::fromLatin1("org.kde.") + binName;
-                request->tolerant_dbus_name = QString::fromLatin1("*.") + binName;
+                request->dbus_name = QStringLiteral("org.kde.") + binName;
+                request->tolerant_dbus_name = QStringLiteral("*.") + binName;
             }
         }
     }
 
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-    // qDebug() << "name=" << request->name << "dbus_name=" << request->dbus_name
+    qCDebug(KLAUNCHER) << "name=" << request->name << "dbus_name=" << request->dbus_name
             << "startup type=" << s_DBusStartupTypeToString[request->dbus_startup_type];
 #endif
 
@@ -957,12 +968,12 @@ KLauncher::slotDequeue()
         if (request->status != KLaunchRequest::Launching) {
             // Request handled.
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-            // qDebug() << "Request handled already";
+            qCDebug(KLAUNCHER) << "Request handled already";
 #endif
             requestDone(request);
             continue;
         }
-    } while (requestQueue.count());
+    } while (!requestQueue.isEmpty());
     bProcessingQueue = false;
 }
 
@@ -1072,8 +1083,7 @@ KLauncher::requestSlave(const QString &protocol,
     QString name = slaveModulePath;
 #endif
 
-    // qDebug() << "KLauncher: launching new slave " << name << " with protocol=" << protocol
-    // << " args=" << arg_list << endl;
+    qCDebug(KLAUNCHER) << "KLauncher: launching new slave" << name << "with protocol=" << protocol << "args=" << arg_list;
 
 #ifdef Q_OS_UNIX
 #ifndef USE_KPROCESS_FOR_KIOSLAVES
@@ -1090,12 +1100,12 @@ KLauncher::requestSlave(const QString &protocol,
 #ifndef USE_KPROCESS_FOR_KIOSLAVES // otherwise we've already done this
         arg_list.prepend(QFile::decodeName(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/kioslave"));
 #endif
-        name = QString::fromLatin1("valgrind");
+        name = QStringLiteral("valgrind");
 
         if (!mSlaveValgrindSkin.isEmpty()) {
             arg_list.prepend(QLatin1String("--tool=") + mSlaveValgrindSkin);
         } else {
-            arg_list.prepend(QLatin1String("--tool=memcheck"));
+            arg_list.prepend(QStringLiteral("--tool=memcheck"));
         }
     }
 #endif
@@ -1113,7 +1123,7 @@ KLauncher::requestSlave(const QString &protocol,
     requestStart(request);
     pid_t pid = request->pid;
 
-//    qDebug() << "Slave launched, pid = " << pid;
+//    qCDebug(KLAUNCHER) << "Slave launched, pid = " << pid;
 
     // We don't care about this request any longer....
     requestDone(request);
@@ -1217,7 +1227,7 @@ KLauncher::slotGotOutput()
 #ifdef USE_KPROCESS_FOR_KIOSLAVES
     QProcess *p = static_cast<QProcess *>(sender());
     QByteArray _stdout = p->readAllStandardOutput();
-    // qDebug() << _stdout.data();
+    qCDebug(KLAUNCHER) << _stdout.data();
 #endif
 }
 
@@ -1226,12 +1236,12 @@ KLauncher::slotFinished(int exitCode, QProcess::ExitStatus exitStatus)
 {
 #ifdef USE_KPROCESS_FOR_KIOSLAVES
     QProcess *p = static_cast<QProcess *>(sender());
-    // qDebug() << "process finished exitcode=" << exitCode << "exitStatus=" << exitStatus;
+    qCDebug(KLAUNCHER) << "process finished exitcode=" << exitCode << "exitStatus=" << exitStatus;
 
     foreach (KLaunchRequest *request, requestList) {
         if (request->process == p) {
 #ifdef KLAUNCHER_VERBOSE_OUTPUT
-            // qDebug() << "found KProcess, request done";
+            qCDebug(KLAUNCHER) << "found KProcess, request done";
 #endif
             if (exitCode == 0  && exitStatus == QProcess::NormalExit) {
                 request->status = KLaunchRequest::Done;
@@ -1251,7 +1261,7 @@ KLauncher::slotFinished(int exitCode, QProcess::ExitStatus exitStatus)
 
 void KLauncher::terminate_kdeinit()
 {
-    // qDebug();
+    qCDebug(KLAUNCHER);
 #ifndef USE_KPROCESS_FOR_KIOSLAVES
     klauncher_header request_header;
     request_header.cmd = LAUNCHER_TERMINATE_KDEINIT;
